import java.awt.*;
import objectdraw.*;

public class NetSpotty extends FrameWindowController implements ClientListener {
	Client network;
	EventTimer splashTimer;
	Text text;
	Animation animation;
	
	public NetSpotty() {
		super(600, 400, "NetSpotty");
	}
	
	public void begin() {
		Network.configure("NetSpotty", ServerListener.ECHO, this);
		
		text = new Text("NetSpotty", 300, 200, Color.BLACK, canvas);
		text.setFontSize(72);
		text.setAlignment(Text.CENTER, Text.CENTER);
		text.setBold(true);
		
		animation = new Animation(Animation.DUSTY, text);
		splashTimer = new EventTimer(animation, "start");
		splashTimer.start(0, 2);
	}
	
	public void onMousePress(Location point) {
		point.translate(-5, -5);
		if(network != null) {
			network.sendMessage(encodeLocation(point));
		} else {
			makeSpot(point);
		}
	}
	
	public void makeSpot(Location point) {
		FilledOval oval = new FilledOval(point, 10, 10, canvas);
		EventTimer timer = new EventTimer(this, "fade", oval, EventTimer.SELF);
		timer.start(3, .05);
	}
	
	public void fade(FilledOval oval, EventTimer timer) {
		int alpha = oval.getColor().getAlpha() - 5;
		
		if(alpha == 0) {
			oval.removeFromCanvas();
			timer.stop();
		} else {
			oval.setColor(new Color(0, 0, 0, alpha));
		}
	}

	public void clientConnected(Client client) { 
		network = client;
		splashTimer.stop();
		animation.cancel();
		text.removeFromCanvas();
	}
	
	public void clientDisconnected(Client client) { 
		network = null;
	}
	
	public void messageReceived(Client client, String message) {
		makeSpot(decodeLocation(message));
	}
	
	public Location decodeLocation(String message) {
		int pos = message.indexOf(",");
		int x = Integer.parseInt(message.substring(0, pos));
		int y = Integer.parseInt(message.substring(pos + 1));
		return new Location(x, y);
	}
	
	public String encodeLocation(Location point) {
		String x = Integer.toString(point.getX());
		String y = Integer.toString(point.getY());
		return x + "," + y;
	}

	public static void main(String[] args) {
		new NetSpotty();
	}
}
