import java.util.LinkedList;
import java.util.Iterator;

import objectdraw.*;

public class SpaceWarsServer implements ServerListener {
	LinkedList<Ship> ships;
	LinkedList<Particle> particles;
	Server network;
	EventTimer timer;
	
	public void update() {
		for(Ship ship : ships) {
			if(ship.fireTime == 0) {
				particles.add(ship.fire());
			}
			ship.fireTime--;
			ship.update();
		}
		
		LinkedList<Particle> dead = new LinkedList<Particle>();
		for(Particle particle : particles) {
			particle.update();
			
			if(checkCollisions(particle)) dead.addLast(particle);
		}
		particles.removeAll(dead);
		
		sendState();
	}
	
	public boolean checkCollisions(Particle particle) {
		if(particle.outOfBounds()) return true;
		
		for(Ship ship : ships) {
			if(ship.collided(particle)) {
				ships.get(particle.owner).size++;
				if(ship.size > 10) ship.size -= ship.size / 10;
				return true;
			}
		}
		return false;
	}
	
	public void sendState() {
		String message = "";
		for(Ship ship : ships) {
			message += ship.encode() + "\n";
		}
		
		for(Particle particle : particles) {
			message += particle.encode() + "\n";
		}
		network.sendMessage(message);
	}

	public void channelClosed(Server server, int channelID, String channelName) { }

	public void channelOpened(Server server, int channelID, String channelName) {
		ships.add(new Ship((int)(Math.random() * 600), (int)(Math.random() * 600), channelID));
		
		// Don't start until we have at least one ship.
		if(!timer.isRunning()) timer.start(.05, .05);
	}

	public void messageReceived(Server server, int channelID, String channelName, String message) {
		if(message.equals("leftPress") || message.equals("rightRelease")) {
			ships.get(channelID).ax--;
		} else if(message.equals("leftRelease") || message.equals("rightPress")) {
			ships.get(channelID).ax++;
		} else if(message.equals("upPress") || message.equals("downRelease")) {
			ships.get(channelID).ay--;
		} else if(message.equals("upRelease") || message.equals("downPress")) {
			ships.get(channelID).ay++;
		} else if(message.equals("spacePress") && ships.get(channelID).fireTime < 0) {
			ships.get(channelID).fireTime = 0;
		} else if(message.equals("spaceRelease")) {
			ships.get(channelID).fireTime = -1;
		}
	}

	public void serverStarted(Server server) {
		ships = new LinkedList<Ship>();
		particles = new LinkedList<Particle>();
		network = server;
		
		timer = new EventTimer(this, "update");
	}

	public void serverStopped(Server server) { }
}
