package legos;
import java.awt.Color;
import java.awt.geom.AffineTransform;
import java.awt.geom.Arc2D;
import java.awt.geom.GeneralPath;

import objectdraw.*;

/**
 * A <code>Hand</code> is a <code>LegoPiece</code> representing the hand of a lego person. It
 * has no subunits.
 * 
 * @author Russell Zahniser (russell@zahniser.net)
 */
public class Hand extends LegoPiece {
	static {
		register();
	}
	
	private static float sin(double arg) {
		return (float)Math.sin(arg * Math.PI / 180.);
	}
	
	private static float cos(double arg) {
		return (float)Math.cos(arg * Math.PI / 180.);
	}
	
	private static void register() {
		float a = 12;
		float ax = a * cos(40), ay = a * sin(40);
		
		AffineTransform t = new AffineTransform();
		t.rotate(-Math.PI * 50 / 180.);

		GeneralPath path = new GeneralPath();
		path.append(t.createTransformedShape(new Arc2D.Double(-5, -3, 10, 6, 180, -180, Arc2D.OPEN)), false);
		path.append(new Arc2D.Double(ax - 6, ay - 12, 12, 24, 130, -140, Arc2D.OPEN), true);
		path.append(new Arc2D.Double(ax - 2, ay - 5, 4, 10, -10, 300, Arc2D.OPEN), true);
		path.append(new Arc2D.Double(ax - 6, ay - 12, 12, 24, 290, -100, Arc2D.OPEN), true);
		path.closePath();
		
		LegoPiece.registerShape(Hand.class, Color.YELLOW, path, path);
	}
	
	/**
	 * Construct a hand, placing it on the given canvas. The hand will be colored yellow.
	 * @param canvas Canvas to place this hand on.
	 */
	public Hand(DrawingCanvas canvas) {
		this(null, canvas);
	}
	
	/**
	 * Construct a hand, placing it on the given canvas.
	 * @param color The color of the hand.
	 * @param canvas Canvas to place this hand on.
	 */
	public Hand(Color color, DrawingCanvas canvas) {
		super(color, canvas);
	}
	
	/**
	 * Construct a hand, but do not make it appear on any canvas
	 * yet. It will be colored yellow. You can make it appear by adding it to a canvas, or 
	 * setting it as one of the hand of an <code>Arm</code> that is already visible on a canvas.
	 */
	public Hand() {
		this(null, null);
	}
	
	/**
	 * Construct a hand, but do not make it appear on any canvas
	 * yet. You can make it appear by adding it to a canvas, or 
	 * setting it as one of the hand of an <code>Arm</code> that is already visible on a canvas.
	 * @param color The color of the hand.
	 */
	public Hand(Color color) {
		super(color, null);
	}
}