package legos;
import java.awt.Color;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Arc2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;

import objectdraw.DrawingCanvas;

/**
 * A <code>Head</code> is a <code>LegoPiece</code> representing the head of a lego person. It
 * has no subunits.
 * 
 * @author Russell Zahniser (russell@zahniser.net)
 */
public class Head extends LegoPiece {
	static {
		LegoPiece.registerShape(Head.class, Color.YELLOW, buildOutline(), buildFill());
	}
	
	private static Shape buildFill() {
		GeneralPath path = new GeneralPath();
		path.append(new Arc2D.Double(0, 5, 20, 10, 180, -90, Arc2D.OPEN), true);
		path.lineTo(10, 0);
		path.lineTo(22, 0);
		path.append(new Arc2D.Double(12, 5, 20, 10, 90, -90, Arc2D.OPEN), true);
		path.append(new Arc2D.Double(12, 30, 20, 10, 0, -60, Arc2D.OPEN), true);
		path.lineTo(27, 44);
		path.lineTo(5, 44);
		path.append(new Arc2D.Double(0, 30, 20, 10, 240, -60, Arc2D.OPEN), true);
		path.closePath();
		return AffineTransform.getTranslateInstance(-16, -44).createTransformedShape(path);
	}
	
	private static Shape buildOutline() {
		GeneralPath path = new GeneralPath();
		// General outline of face
		path.append(new Arc2D.Double(12, 5, 20, 10, 90, -90, Arc2D.OPEN), false);
		path.append(new Arc2D.Double(12, 30, 20, 10, 0, -90, Arc2D.OPEN), true);
		path.append(new Arc2D.Double(0, 30, 20, 10, 270, -90, Arc2D.OPEN), true);
		path.append(new Arc2D.Double(0, 5, 20, 10, 180, -90, Arc2D.OPEN), true);
		
		// Eyes and mouth
		path.append(new Ellipse2D.Double(17, 14, 2, 2), false);
		path.append(new Ellipse2D.Double(25, 14, 2, 2), false);
		path.append(new Arc2D.Double(13, 22, 18, 8, 210, 120, Arc2D.OPEN), false);
		
		// top bump
		path.append(new Rectangle2D.Double(10, 0, 12, 5), false);
		
		// neck
		path.moveTo(27, 35 + (float)Math.sqrt(3) * 2.5f);
		path.lineTo(27, 44);
		path.lineTo(5, 44);
		path.lineTo(5, 35 + (float)Math.sqrt(3) * 2.5f);
		return AffineTransform.getTranslateInstance(-16, -44).createTransformedShape(path);
	}
	
	
	/**
	 * Construct a head, placing it on the given canvas. The head will be colored yellow.
	 * @param canvas Canvas to place this head on.
	 */
	public Head(DrawingCanvas canvas) {
		this(null, canvas);
	}
	
	/**
	 * Construct a head, placing it on the given canvas.
	 * @param color The color of the head.
	 * @param canvas Canvas to place this head on.
	 */
	public Head(Color color, DrawingCanvas canvas) {
		super(color, canvas);
	}
	
	/**
	 * Construct a head, but do not make it appear on any canvas
	 * yet. It will be colored yellow. You can make it appear by adding it to a canvas, or 
	 * setting it as one of the head of a <code>Body</code> that is already visible on a canvas.
	 */
	public Head() {
		this(null, null);
	}
	
	/**
	 * Construct a head, but do not make it appear on any canvas
	 * yet. You can make it appear by adding it to a canvas, or 
	 * setting it as one of the head of a <code>Body</code> that is already visible on a canvas.
	 * @param color The color of the head.
	 */
	public Head(Color color) {
		super(color, null);
	}
}
