package legos;
import java.awt.Color;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Arc2D;
import java.awt.geom.GeneralPath;

import objectdraw.DrawingCanvas;

/**
 * A <code>Leg</code> is a <code>LegoPiece</code> representing the leg of a lego person. It
 * has no subunits.
 * 
 * @author Russell Zahniser (russell@zahniser.net)
 */
public class Leg extends LegoPiece {
	static {
		LegoPiece.registerShape(Leg.class, Color.BLUE, buildOutline(), buildFill());
	}
	
	private static Shape buildFill() {
		GeneralPath path = new GeneralPath();
		path.append(new Arc2D.Double(0, 0, 12, 40, 180, -90, Arc2D.OPEN), false);
		path.append(new Arc2D.Double(20, 0, 12, 40, 90, -150, Arc2D.OPEN), true);
		path.lineTo(29, 60);
		path.lineTo(32, 60);
		path.lineTo(32, 68);
		path.lineTo(0, 68);
		path.closePath();
		return AffineTransform.getTranslateInstance(-16, -20).createTransformedShape(path);
	}
	
	private static Shape buildOutline() {
		GeneralPath path = new GeneralPath();
		
		// Side of leg
		path.append(new Arc2D.Double(0, 0, 12, 40, 180, -240, Arc2D.OPEN), false);
		path.lineTo(9, 60);
		path.lineTo(12, 60);
		path.lineTo(12, 68);
		path.lineTo(0, 68);
		path.closePath();
		
		// Front of leg
		path.moveTo(6, 0);
		path.append(new Arc2D.Double(20, 0, 12, 40, 90, -150, Arc2D.OPEN), true);
		path.lineTo(29, 60);
		path.lineTo(32, 60);
		path.lineTo(32, 68);
		path.lineTo(12, 68);
		
		path.moveTo(9, 60);
		path.lineTo(29, 60);
		return AffineTransform.getTranslateInstance(-16, -20).createTransformedShape(path);
	}
	
	/**
	 * Construct a leg, placing it on the given canvas. The leg will be colored blue.
	 * @param canvas Canvas to place this leg on.
	 */
	public Leg(DrawingCanvas canvas) {
		this(null, canvas);
	}
	
	/**
	 * Construct a leg, placing it on the given canvas.
	 * @param color The color of the leg.
	 * @param canvas Canvas to place this leg on.
	 */
	public Leg(Color color, DrawingCanvas canvas) {
		super(color, canvas);
	}
	
	/**
	 * Construct a leg, but do not make it appear on any canvas
	 * yet. It will be colored blue. You can make it appear by adding it to a canvas, or 
	 * setting it as one of the legs of a <code>Legs</code> that is already visible on a canvas.
	 */
	public Leg() {
		this(null, null);
	}
	
	/**
	 * Construct a leg, but do not make it appear on any canvas
	 * yet. You can make it appear by adding it to a canvas, or 
	 * setting it as one of the legs of a <code>Legs</code> that is already visible on a canvas.
	 * @param color The color of the leg.
	 */
	public Leg(Color color) {
		super(color, null);
	}
}
