package objectdraw;

/**
 * The <code>ClientListener</code> interface specifies which methods an object needs
 * to have in order to be able to function as a client listener. The client listener
 * will be notified when the client connects to the server, when it disconnects or
 * is disconnected, and when a message is received from the server.
 * <br><br>
 * To specify what object will be your <code>ClientListener</code>, use the
 * <code>Network.configure()</code> method.
 * 
 * @see Network#configure(String, ServerListener, ClientListener)
 * @author Russell Zahniser (russell@zahniser.net)
 */
public interface ClientListener {
	/**
	 * This method will be called when the client connects to the server.
	 * @param client The client
	 */
	public void clientConnected(Client client);
	/**
	 * This method will be called when a message is received from the server.
	 * @param client The client
	 * @param message Message that was received
	 */
	public void messageReceived(Client client, String message);
	/**
	 * This method will be called when the client disconnects or is disconnected
	 * from the server.
	 * @param client The client
	 */
	public void clientDisconnected(Client client);
}
