package objectdraw;

import java.applet.Applet;
import java.applet.AppletContext;
import java.applet.AppletStub;
import java.applet.AudioClip;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

/**
 * @deprecated internal use only
 * @author Russell Zahniser (russell@zahniser.net)
 */
public class ControllerFrame extends JFrame implements AppletStub, AppletContext {
	private static ArrayList frames = new ArrayList();
    private static boolean showingRulers = false;
    private static boolean showingGrids = false;
	
	private static final Color RULER_COLOR = new Color(50, 50, 50);	
	private static final Color RULER_BACKGROUND = new Color(200, 200, 200);
	private static final Font RULER_FONT = Font.decode("Times-BOLD-12");
	private static final Color TICK_COLOR = Color.BLUE;
	
	static {
		Preferences prefs = Preferences.userRoot().node("/com/zahniser/objectdraw");
		
		String p = prefs.get("view", "");
		showingRulers = (p.indexOf("R") != -1);
		showingGrids = (p.indexOf("G") != -1);
		
		Runtime.getRuntime().addShutdownHook(new Thread() {
			public void run() {
				Preferences prefs = Preferences.userRoot().node("/com/zahniser/objectdraw");
				prefs.put("view", (showingRulers ? (showingGrids ? "RG" : "R") : (showingGrids ? "G" : "")));
				try {
					prefs.flush();
				} catch (BackingStoreException e) {
				}
			}
		});
	}
	
    private static class AppletEnumerator implements Enumeration {
    	private ArrayList framesCopy;
    	private int index;
    	
    	public AppletEnumerator() {
    		framesCopy = new ArrayList(frames);
    		index = 0;
    	}
        
        public Object nextElement() {
            if(hasMoreElements()) {
                return ((ControllerFrame)frames.get(index++)).applet;
            } else {
                return null;
            }
        }
        
        public boolean hasMoreElements() {
            return index < framesCopy.size();
        }
    }
    
    // This ruler extends horizontally along the top of the window. It displays pixels in increments
    // of 10, with larger, labeled marks every 50, and shows the current x location of the mouse when
    // the mouse is over the controller.
    private class HorizontalRuler extends JComponent {
    	public void paintComponent(Graphics graphics) {
    		Graphics2D g = (Graphics2D)graphics;
    		
    		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    		g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
            g.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_ON);
            
            int w = getWidth();
            int h = getHeight();
            g.setColor(RULER_BACKGROUND);
            g.fillRect(0, 0, w, h);
            
            g.setColor(RULER_COLOR);
            g.setFont(RULER_FONT);
            FontMetrics fm = g.getFontMetrics();
            
            for(int i = 0, m = 0; i < w; i += 10, m--) {
            	if(m == 0) {
                	g.drawLine(i, 22, i, 30);
            		m = 5;
            	} else {
                	g.drawLine(i, 25, i, 30);
            	}
            }
            g.drawLine(0, 29, w, 29);
            
            for(int i = 50; i < w; i += 50) {
            	String s = Integer.toString(i);
            	g.drawString(s, i - fm.stringWidth(s) / 2, 20);
            }
            
            if(corner.loc != null) {
            	g.setColor(TICK_COLOR);
            	int x = corner.loc.getX();
            	g.drawLine(x, 20, x, 30);
            }
    	}
    	
    	public Dimension getPreferredSize() {
    		Dimension d = applet.getPreferredSize();
    		d.height = 30;
    		return d;
    	}
    }
    
    // This ruler extends vertically along the left of the window. It displays pixels in increments
    // of 10, with larger, labeled marks every 50, and shows the current y location of the mouse when
    // the mouse is over the controller.
    private class VerticalRuler extends JComponent {
    	public void paintComponent(Graphics graphics) {
    		Graphics2D g = (Graphics2D)graphics;
    		
    		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    		g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
            g.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_ON);
            
            int w = getWidth();
            int h = getHeight();
            g.setColor(RULER_BACKGROUND);
            g.fillRect(0, 0, w, h);
            
            g.setColor(RULER_COLOR);
            g.setFont(RULER_FONT);
            FontMetrics fm = g.getFontMetrics();
            
            for(int i = 0, m = 0; i < h; i += 10, m--) {
            	if(m == 0) {
                	g.drawLine(22, i, 30, i);
            		m = 5;
            	} else {
                	g.drawLine(25, i, 30, i);
            	}
            }
            g.drawLine(29, 0, 29, h);
            
            int d = fm.getAscent() / 2;
            for(int i = 50; i < h; i += 50) {
            	String s = Integer.toString(i);
            	g.drawString(s, 20 - fm.stringWidth(s), i + d);
            }
            
            if(corner.loc != null) {
            	g.setColor(TICK_COLOR);
            	int y = corner.loc.getY();
            	g.drawLine(20, y, 30, y);
            }
    	}
    	
    	public Dimension getPreferredSize() {
    		Dimension d = applet.getPreferredSize();
    		d.width = 30;
    		return d;
    	}
    }
    
    // This component displays the current (x, y) position of the mouse whenever it is over the controller.
    private class LocationView extends JComponent implements MouseListener, MouseMotionListener {
		private Location loc;
		private Component pressedComponent;
		
		public Dimension getPreferredSize() {
			return new Dimension(30, 30);
		}
		
    	public void paintComponent(Graphics graphics) {
    		Graphics2D g = (Graphics2D)graphics;
    		
    		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    		g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
            g.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_ON);
            
            int w = getWidth();
            int h = getHeight();
            g.setColor(RULER_BACKGROUND);
            g.fillRect(0, 0, w, h);

    		if(loc == null) return;
    		
            g.setColor(RULER_COLOR);
            g.setFont(RULER_FONT);
            FontMetrics fm = g.getFontMetrics();
            
            String s = "(" + loc.getX() + ",";
            g.drawString(s, 2, fm.getAscent());
            s = loc.getY() + ")";
            g.drawString(s, w - 2 - fm.stringWidth(s), fm.getAscent() + fm.getHeight());
    	}
    	
    	public void mouseClicked(MouseEvent event) { 
    		redispatchEvent(event);
    	}
		public void mousePressed(MouseEvent event) { 
    		redispatchEvent(event);
        }
		public void mouseReleased(MouseEvent event) { 
    		redispatchEvent(event);
        }

		public void mouseEntered(MouseEvent event) {
			updateLocation(event); 
    		redispatchEvent(event);
		}

		public void mouseExited(MouseEvent event) {
			updateLocation(null);
    		redispatchEvent(event);
		}
		
		public void mouseDragged(MouseEvent event) {
			if(event.getX() >= 0 && event.getX() < applet.getWidth() &&
					event.getY() >= 0 && event.getY() < applet.getHeight()) {
				updateLocation(event);
			}
    		redispatchEvent(event);
		}

		public void mouseMoved(MouseEvent event) {
			updateLocation(event);
    		redispatchEvent(event);
		}
		
		private void updateLocation(MouseEvent event) {
			if(event == null) {
				loc = null;
			} else {
				loc = new Location(event.getX(), event.getY());
			}
			repaint();
			left.repaint();
			top.repaint();
		}
		
		private void redispatchEvent(MouseEvent event) {
			Component target;
			if(event.getID() == MouseEvent.MOUSE_DRAGGED) {
				target = pressedComponent;
			} else {
				target = SwingUtilities.getDeepestComponentAt(
						applet.getContentPane(), event.getX(), event.getY());
			}
			if(target == null) return;
			
			if(event.getID() == MouseEvent.MOUSE_PRESSED) {
				pressedComponent = target;
			} else if(event.getID() == MouseEvent.MOUSE_RELEASED) {
				pressedComponent = null;
			}
			
			MouseEvent redispatch = SwingUtilities.convertMouseEvent(
                    applet.getGlassPane(), event, target);
			target.dispatchEvent(redispatch);
		}
    }
    
    public void showRulers() {
		getContentPane().remove(applet);
		
		getContentPane().setLayout(new GridBagLayout());
        
        GridBagConstraints g = new GridBagConstraints();
        g.gridy = 1;
        g.gridx = 0;
        g.fill = GridBagConstraints.VERTICAL;
        getContentPane().add(left, g);
        g.gridy = 0;
        g.fill = GridBagConstraints.NONE;
        getContentPane().add(corner, g);
        g.gridx = 1;
        g.fill = GridBagConstraints.HORIZONTAL;
        getContentPane().add(top, g);
        g.gridy = 1;
        g.weightx = 1;
        g.weighty = 1;
        g.fill = GridBagConstraints.BOTH;
        getContentPane().add(applet, g);
        
        applet.getGlassPane().addMouseListener(corner);
        applet.getGlassPane().addMouseMotionListener(corner);
        
     	fixSize();
    }
    
    public void hideRulers() {
  		getContentPane().remove(applet);
  		getContentPane().remove(left);
  		getContentPane().remove(top);
  		getContentPane().remove(corner);
  	    getContentPane().setLayout(new BorderLayout());
		getContentPane().add(applet, BorderLayout.CENTER);
		
        applet.getGlassPane().removeMouseListener(corner);
        applet.getGlassPane().removeMouseMotionListener(corner);
        
		fixSize();
    }
    
    public static void setRulers(boolean show) {
    	if(showingRulers == show) return;
    	
    	showingRulers = show;
	
		for(int i = 0; i < frames.size(); i++) {
			ControllerFrame frame = (ControllerFrame)frames.get(i);
			
			if(show) frame.showRulers();
			else frame.hideRulers();
			
			frame.rulersItem.setSelected(show);
		}
    }
    
    public static void setGrids(boolean show) {
    	if(showingGrids == show) return;
    	
    	showingGrids = show;
    	
		for(int i = 0; i < frames.size(); i++) {
			ControllerFrame frame = (ControllerFrame)frames.get(i);
			
			if(show) frame.applet.showGrid();
			else frame.applet.hideGrid();
			
			frame.gridItem.setSelected(show);
		}
    }
    
    public void fixSize() {
    	Dimension d = applet.getPreferredSize();
    	appletResize(d.width, d.height);
    }
    
    private Controller applet;
    private VerticalRuler left;
    private HorizontalRuler top;
    private LocationView corner;
    private boolean active = false;
    private HashMap streams = new HashMap();
    private int width;
    private int height;
    private MouseInterpreter key;
    private JCheckBoxMenuItem rulersItem;
    private JCheckBoxMenuItem gridItem;

    public ControllerFrame(String title, Controller myController) {
        super(title);
        
        setContentPane(new JPanel());
         
        applet = myController;
        
        // The Controller should only have a grid glass pane if it is in a ControllerFrame
        applet.makeGrid();
        
        corner = new LocationView();
        left = new VerticalRuler();
        top = new HorizontalRuler();
        
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        buildMenus();
        
        if(showingRulers) {
        	showRulers();
        } else {
        	hideRulers();
        }
        
        if(showingGrids) {
        	applet.showGrid();
        } else {
        	applet.hideGrid();
        }
        
        Network.frameCreated(this);
        
        // tell the applet to lay itself out
        validate();

        // set up actions to take when window closes
        addWindowListener(new WindowAdapter() {
        	public void windowActivated(WindowEvent e) {
        		if(applet instanceof WindowController) {
        			((WindowController)applet).canvas.requestFocus();
        		}
        	}
        	
            public void windowClosed(WindowEvent e) {
                // shut down the applet
                active = false;
                applet.stop();
                applet.destroy();
                
                System.exit(0);
            }
        });
        
        applet.setStub(this);
        active = true;
        
        frames.add(this);
        
        setVisible(true);
    }
    
    private void buildMenus() {
        boolean mac = false;
        try {
			mac = System.getProperty("os.name").indexOf("Mac") != -1;
        } catch(SecurityException e) {
        }
        
        JMenuBar menuBar = new JMenuBar();

        if(!mac) {
			JMenu menu = new JMenu("File");
			menuBar.add(menu);
			
			JMenuItem quit = new JMenuItem("Exit");
			quit.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
			        active = false;
			        applet.stop();
			        applet.destroy();
			        
			        hide();
			        dispose();
			        
			        System.exit(0);
				}
			});
			menu.add(quit);
		}
        
        JMenu viewMenu = new JMenu("View");
        menuBar.add(viewMenu);
        
        rulersItem = new JCheckBoxMenuItem("Show Rulers", showingRulers);
		rulersItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
		        setRulers(!showingRulers);
			}
		});
		viewMenu.add(rulersItem);
        
        gridItem = new JCheckBoxMenuItem("Show Grid", showingGrids);
        gridItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setGrids(!showingGrids);
			}
		});
		viewMenu.add(gridItem);
		
		setJMenuBar(menuBar);
    }
   
    public boolean isActive() {
        return active;
    }
     
    /**
     * This is here to help out the applet. You don't need to use it.
     */
    public void appletResize(int width, int height) {
    	Dimension d = new Dimension(width, height);
    	if(showingRulers) {
    		d.width += 30;
    		d.height += 30;
    	}
    	((JPanel)getContentPane()).setPreferredSize(d);
    	pack();
    }
    
    /**
     * This is here to help out the applet. You don't need to use it.
     */
    public AppletContext getAppletContext()
    {
        return this;
    }
    
    /**
     * This is here to help out the applet. You don't need to use it.
     */
    public String getParameter(String name)
    {
        return null;
    }
    
    /**
     * This is here to help out the applet. You don't need to use it.
     */
    public URL getCodeBase()
    {
        try {
            return new File(".").toURI().toURL();
        } catch (java.net.MalformedURLException mue) {
            return null;
        }
    }
    
    /**
     * This is here to help out the applet. You don't need to use it.
     */
    public URL getDocumentBase()
    {
        return getCodeBase();
    }
    
    /**
     * Return the applet
     */
    public Applet getApplet(String name) {
        return applet;
    }
    
    /**
     * Return an enumeration containing the applet
     */
    public Enumeration getApplets() {
        return new AppletEnumerator();
    }
        
    /**
     * Return an audio clip from the given URL
     */
    public AudioClip getAudioClip(URL url) {
		return Applet.newAudioClip(url);
    }
    
    /**
     * Return an image from the given URL
     */
    public Image getImage(URL url) {
        return Toolkit.getDefaultToolkit().getImage(url);
    }
    
    /**
     * Return the input stream associated with the given key
     */
    public InputStream getStream(String key) {
        return (InputStream)streams.get(key);
    }
    
    /**
     * Return an iterator over all the stream keys
     */
    public Iterator getStreamKeys() {
        return streams.keySet().iterator();
    }
    
    /**
     * Associate the given stream with the given key
     */
    public void setStream(String key, InputStream stream) {
        streams.put(key, stream);
    }
    
    public void showDocument(URL url) {
    }
    
    public void showDocument(URL url, String target) {
    }
    
    public void showStatus(String status) {
    }
}
