package objectdraw;

import javax.swing.SwingUtilities;

/**
 * <code>FrameController</code> is a <code>Controller</code> that automatically pops up in its
 * own frame (window) when it is constructed. This allows you, in a program like BlueJ or
 * DrJava, to pop up a window instantly by creating an object, rather than having to create
 * it and then tell it to <code>startController()</code>. In these programs, you should make
 * your <code>Controller</code> classes extend <code>FrameController</code> instead.
 * 
 * @author Russell Zahniser (russell@zahniser.net)
 */
public class FrameController extends Controller {
	/**
	 * Construct a controller of size 400 by 400, which will immediately appear in its own window.
	 */
	public FrameController() {
		this(400, 400, "ObjectDraw");
	}
	
	/**
	 * Construct a controller of the specified size, which will appear in its own window.
	 * @param width Width of the controller
	 * @param height Height of the controller
	 */
	public FrameController(int width, int height) {
		this(width, height, "ObjectDraw");
	}
	/**
	 * Construct a controller of size 400 by 400, which will immediately appear in its own window.
	 * @param name Name of window
	 */
	public FrameController(String name) {
		this(400, 400, name);
	}
	
	/**
	 * Construct a controller of the specified size, which will appear in its own window.
	 * @param width Width of the controller
	 * @param height Height of the controller
	 * @param name Name of window
	 */
	public FrameController(int width, int height, String name) {
		final String windowName = name;
		final int w = width, h = height;
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				startController(w, h, windowName);
			}
		});
	}
}
