package objectdraw;

/**
 * A <code>RandomDoubleGenerator</code> will produce a string of random <code>double</code>s that
 * are in the range it was created to produce. Each number x will satisfy 
 * <code>min <= x < max</code>, and the numbers will be distributed evenly and unpredictably
 * within that range.
 * <br><br>
 * You can think of the generator as being like a set of dice, which I can roll over and over
 * again (with <code>nextValue()</code>) to get different results each time. So, if I construct
 * a <code>new RandomIntGenerator(1, 10)</code> it might produce 8.341, then 1.792, and so on.
 * 
 * @author Russell Zahniser (russell@zahniser.net)
 */
public class RandomDoubleGenerator {
	private double min, max;
	
	/**
	 * Construct a random number generator that will deliver <code>double</code>s in the given
	 * range.
	 * @param min Minimum value that may be produced
	 * @param max Maximum value that may be produced
	 */
	public RandomDoubleGenerator(double min, double max) {
		this.min = min;
		this.max = max;
	}
	
	/**
	 * Return another random number
	 * @return Another number in the range of this generator
	 */
	public double nextValue() {
		return Math.random() * (max - min) + min;
	}
}
