package objectdraw;

/**
 * A <code>RandomIntGenerator</code> will produce a string of random <code>int</code>s that
 * are in the range it was created to produce. Each number x will satisfy 
 * <code>min <= x <= max</code>, and the numbers will be distributed evenly and unpredictably
 * within that range.
 * <br><br>
 * You can think of the generator as being like a set of dice, which I can roll over and over
 * again (with <code>nextValue()</code>) to get different results each time. So, if I construct
 * a <code>new RandomIntGenerator(1, 10)</code> it might produce 3, then 7, then 5, and so on.
 * 
 * @author Russell Zahniser (russell@zahniser.net)
 */
public class RandomIntGenerator {
	private int min, max;
	
	/**
	 * Construct a random number generator that will deliver <code>int</code>s in the given
	 * range.
	 * @param min Minimum value that may be produced
	 * @param max Maximum value that may be produced
	 */
	public RandomIntGenerator(int min, int max) {
		this.min = min;
		this.max = max;
	}
	
	/**
	 * Return another random number
	 * @return Another number in the range of this generator
	 */
	public int nextValue() {
		return (int)(Math.random() * (max - min + 1)) + min;
	}
}
